#define STRICT
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <windows.h>
#include <tchar.h>
#include <comdef.h>
#include <ddraw.h>

#pragma comment (lib, "ddraw.lib")
#pragma comment (lib, "dxguid.lib")

_COM_SMARTPTR_TYPEDEF(IDirectDraw7, IID_IDirectDraw7);
_COM_SMARTPTR_TYPEDEF(IDirectDrawSurface7, IID_IDirectDrawSurface7);

HWND InitWindow(HINSTANCE instance_handle);

int PASCAL _tWinMain(HINSTANCE hinst, HINSTANCE, LPTSTR, int nShowCmd)
{
  if (SUCCEEDED(::CoInitialize(NULL))) {
    const HWND window_handle = InitWindow(hinst);

    if (window_handle) {
      ::ShowWindow(window_handle, nShowCmd);

      HRESULT hr = S_OK;

      IDirectDraw7Ptr dd;
      hr = ::DirectDrawCreateEx(NULL, reinterpret_cast<VOID**>(&dd), IID_IDirectDraw7, NULL);
      hr = dd->SetCooperativeLevel(window_handle, DDSCL_NORMAL);

      IDirectDrawSurface7Ptr front_buffer;
      DDSURFACEDESC2 surface_desc;
      ZeroMemory(&surface_desc, sizeof(surface_desc));
      surface_desc.dwSize = sizeof(surface_desc);
      surface_desc.dwFlags = DDSD_CAPS;
      surface_desc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
      hr = dd->CreateSurface(&surface_desc, &front_buffer, NULL);

      {
        DDSURFACEDESC2 lock_desc;
        ZeroMemory(&lock_desc, sizeof(lock_desc));
        lock_desc.dwSize = sizeof(lock_desc);

        // DirectDraw Lock
        // Windows 7: Applications manifested for Windows 7 cannot call Lock API in DDRAW to lock 
        //   the primary Desktop video buffer. Doing so will result in error, and NULL pointer for
        //   the primary will be returned. This behavior is enforced even if Desktop Window Manager
        //   Composition is not turned on. Windows 7 compatible applications must not lock the
        //   primary video buffer to render.
        // Windows Vista (default): Applications will be able to acquire a lock on the primary video
        //   buffer as legacy applications depend on this behavior. Running the application turns off
        //   Desktop Window Manager.
        hr = front_buffer->Lock(NULL, &lock_desc, DDLOCK_WAIT, NULL);
        if (SUCCEEDED(hr)) {
          front_buffer->Unlock(NULL);
          ::OutputDebugString(_T("Lock Succeeded\n"));
          if (lock_desc.lpSurface == NULL) {
            ::OutputDebugString(_T("Retrieved pointer is NULL\n"));
          }
        } else {
          ::OutputDebugString(_T("Lock failed\n"));
        }
      }

      MSG msg;
      while (::GetMessage(&msg, NULL, 0, 0)) {
        ::TranslateMessage(&msg);
        ::DispatchMessage(&msg);
      }
    }
    ::CoUninitialize();
  }
  return 0;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
  switch (uMsg) {
    case WM_DESTROY:
      ::PostQuitMessage(0);
      break;
  }
  return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

HWND InitWindow(HINSTANCE instance_handle) {
  const TCHAR *class_name = _T("Windows 7 Compatiblity Test");

  WNDCLASS wc      = {0};
  wc.style         = 0;
  wc.lpfnWndProc   = WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = instance_handle;
  wc.hIcon         = NULL;
  wc.hCursor       = ::LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = reinterpret_cast<HBRUSH>(COLOR_WINDOW + 1);
  wc.lpszMenuName  = NULL;
  wc.lpszClassName = class_name;

  ::RegisterClass(&wc);

  return ::CreateWindowEx(
    0, class_name, TEXT("Test Window"), WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
    NULL, NULL, instance_handle, NULL);
}
